<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\BTL\Services\FasesTemplateService;

class FasesTemplateController extends Crud {

    protected $dataSourceName = 'des_fasetemplate';
    protected $dataSourceName2 = 'dadosFaseTemplate';
    protected $dataSourceName3 = 'minNrOrdem';
    protected $dataSourceName4 = 'maxNrOrdem';

    protected $parameterBagColumns = array('NRORG', 'NRTEMPLATE');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, FasesTemplateService $fasesTemplateService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->fasesTemplateService = $fasesTemplateService;
        parent::__construct($dataSourceManager);
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName2);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $buscar = array();
            foreach($dataSet->getRows() as &$row) {
                if(array_key_exists('NRDURACAO', $row)){
                    $row['NRDURACAO'] = DateUtil::getHoraStringDeFloat($row['NRDURACAO']);
                }
                $buscar[] = $row;
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('buscarDadosFaseTemplate', $buscar));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscarNrOrdemMin (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName3);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscarNrOrdemMax (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName4);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarDadosFaseTemplate (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if(array_key_exists('FASES', $row)){
                    foreach ($row["FASES"] as $rows) {
                        if(array_key_exists('NRDURACAO', $rows)){
                            $rows['NRDURACAO'] = DateUtil::getHoraFloatDeString($rows['NRDURACAO']);
                        }

                        if(array_key_exists('__is_new', $row)){
                            $rows['__is_new'] = $row['__is_new'];
                        } 

                        if(!array_key_exists('IDINATIVARPROCESSO', $row)){
                            $rows['IDINATIVARPROCESSO'] = 'N';
                        }

                        if(!array_key_exists('NRORDEM', $rows)){
                            $rows['NRORDEM'] = $this->fasesTemplateService->getNextId($row['NRORG'], $row['NRTEMPLATE']);
                        }

                        if($row['__is_new'] === true){
                            $rows['NRDURACAO'] = null;
                            $rows['NRPESO'] = null;
                        }
                        $rows['NRTEMPLATE'] = $row['NRTEMPLATE'];
                        if(array_key_exists('NRFASETEMPLATE', $rows)){
                            if($rows['NRFASETEMPLATE'] === '' ){
                                $rows['NRFASETEMPLATE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_FASETEMPLATE', 1, 12, true);
                            }
                        } else {
                            $rows['NRFASETEMPLATE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_FASETEMPLATE', 1, 12, true);
                        }

                        if(!array_key_exists('DTINCLUSAO', $rows)){
                          $rows['DTINCLUSAO'] = date('d/m/Y');
                        } else {
                            $explode = explode(" ", $rows['DTINCLUSAO']);
                            if(isset($explode[1])){
                                $rows['DTINCLUSAO'] = $explode[0];
                            }
                        }

                        if(!array_key_exists('DTULTATU', $rows)){
                          $rows['DTULTATU'] = date('d/m/Y');
                        } else {
                            $explode = explode(" ", $rows['DTULTATU']);
                            if(isset($explode[1])){
                                $rows['DTULTATU'] = $explode[0];
                            }
                        }

                        if(!array_key_exists('NRORGINCLUSAO', $rows)){
                            $rows['NRORGINCLUSAO'] = $rows['NRORG'];
                        }

                        if(!array_key_exists('NRORGULTATU', $rows)){
                            $rows['NRORGULTATU'] = $rows['NRORG'];
                        }

                        if(!array_key_exists('CDOPERINCLUSAO', $rows)){
                            $rows['CDOPERINCLUSAO'] = $rows['CDOPERADOR_LOG'];
                        }

                        if(!array_key_exists('CDOPERULTATU', $rows)){
                            $rows['CDOPERULTATU'] = $rows['CDOPERADOR_LOG'];
                        }

                        if(!array_key_exists('NRDURACAO', $rows)){
                            $rows['NRDURACAO'] = null;
                        }

                        if(!array_key_exists('NRPESO', $rows)){
                            $rows['NRPESO'] = null;
                        }

                        if(!array_key_exists('NRFASERELACIONADA', $rows))
                            $rows['NRFASERELACIONADA'] = null;

                        if(!array_key_exists('DTINICIOFASE', $rows)){
                            $rows['DTINICIOFASE'] = null;
                        } else {
                            $explode = explode(" ", $rows['DTINICIOFASE']);
                            if(isset($explode[1])){
                                $rows['DTINICIOFASE'] = $explode[0];
                            }
                        }

                        if(!array_key_exists('DTFIMFASE', $rows)){
                            $rows['DTFIMFASE'] = null;
                        } else {
                            $explode = explode(" ", $rows['DTFIMFASE']);
                            if(isset($explode[1])){
                                $rows['DTFIMFASE'] = $explode[0];
                            }
                        }

                        $requestSave = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_fasetemplate', array($rows))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::save($requestSave, $response);
                    }
                } else {
                    if(array_key_exists('NRDURACAO', $row)){
                        $row['NRDURACAO'] = DateUtil::getHoraFloatDeString($row['NRDURACAO']);
                    }

                    if(!array_key_exists('IDINATIVARPROCESSO', $row)){
                        $row['IDINATIVARPROCESSO'] = 'N';
                    }

                    if(!array_key_exists('DTINCLUSAO', $row)){
                      $row['DTINCLUSAO'] = date('d/m/Y');
                    } else {
                        $explode = explode(" ", $row['DTINCLUSAO']);
                        if(isset($explode[1])){
                            $row['DTINCLUSAO'] = $explode[0];
                        }
                    }

                    if(!array_key_exists('NRORDEM', $row)){
                        $row['NRORDEM'] = $this->fasesTemplateService->getNextId($row['NRORG'], $row['NRTEMPLATE']);
                    }

                    if(!array_key_exists('DTULTATU', $row)){
                      $row['DTULTATU'] = date('d/m/Y');
                    } else {
                        $explode = explode(" ", $row['DTULTATU']);
                        if(isset($explode[1])){
                            $row['DTULTATU'] = $explode[0];
                        }
                    }

                    if(!array_key_exists('NRORGINCLUSAO', $row)){
                        $row['NRORGINCLUSAO'] = $row['NRORG'];
                    }

                    if(!array_key_exists('NRORGULTATU', $row)){
                        $row['NRORGULTATU'] = $row['NRORG'];
                    }

                    if(!array_key_exists('CDOPERINCLUSAO', $row)){
                        $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    }

                    if(!array_key_exists('CDOPERULTATU', $row)){
                        $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    }

                    if(!array_key_exists('NRDURACAO', $row)){
                        $row['NRDURACAO'] = null;
                    }

                    if(!array_key_exists('NRPESO', $row)){
                        $row['NRPESO'] = null;
                    }

                    if(!array_key_exists('NRFASERELACIONADA', $row))
                        $row['NRFASERELACIONADA'] = null;

                    if(!array_key_exists('DTINICIOFASE', $row)){
                        $row['DTINICIOFASE'] = null;
                    } else {
                        $explode = explode(" ", $row['DTINICIOFASE']);
                        if(isset($explode[1])){
                            $row['DTINICIOFASE'] = $explode[0];
                        }
                    }

                    if(!array_key_exists('DTFIMFASE', $row)){
                        $row['DTFIMFASE'] = null;
                    } else {
                        $explode = explode(" ", $row['DTFIMFASE']);
                        if(isset($explode[1])){
                            $row['DTFIMFASE'] = $explode[0];
                        }
                    }
                    parent::save($request, $response);
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}