<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Bradesco;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\MsgCalculo;
use HCM\FPA\Entity\FpaFormcnabEntity;
use HCM\FPA\Entity\FpaCnabmagnetEntity;

class Cnab240 {
    protected $nrorg;
    protected $competencia;
    protected $nrmodalidade;
    protected $nrformcnab;
    protected $idativo;
    protected $idprocvinculo;
    protected $idgerameiomag;
    protected $idteste;
    protected $nmformcnab;
    protected $dtpagamento;
    protected $nrhierarqorg;
    protected $dsordem;
    protected $dsinicio;
    protected $dsfim;
    protected $nrtipomoviment;
    protected $codconvenio;
    protected $nreventom;
    protected $nreventom__label;
    protected $nrvinculom;
    protected $cdmatricula;
    protected $idtpinschierarq;
    protected $nrpessoarespons;
    protected $nrpessoarespons__label;
    protected $nrestrutrespons;
    protected $idtpinscrespons;
    protected $idutildtpgtoitem;
    protected $idignoraocorrencia;
    protected $nrCalculo;

    protected $vrtotal;

    protected $vinculos;
    protected $estruturas;

    protected $nomeArquivo = '';
    protected $caminhoRelativo = '';
    protected $caminhoRelCompleto = '';
    protected $nrSeqArquvio;

    protected $cdOperador;

    public function __construct($entityManager, $hcmParameters, $connection,  $params){
        
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->nrorg = FPA::getNrorgTrabalho();
        $this->competencia = FPA::getDtCompetencia();
        $this->nrmodalidade = FPA::getNrModalidadeCalculo();
        $this->nrformcnab = $params['NRFORMCNAB'];
        $this->idativo = $params['IDATIVO'];
        $this->idprocvinculo = $params['IDPROCVINCULO'];
        $this->idgerameiomag = $params['IDGERAMEIOMAG'];
        $this->idteste = $params['IDTESTE'];
        $this->nmformcnab = $params['NMFORMCNAB'];
        $this->dtpagamento = $params['DTPAGAMENTO'];
        $this->nrhierarqorg = $params['NRHIERARQORG'];
        $this->dsordem = $params['DSORDEM'];
        $this->dsinicio = $params['DSINICIO'];
        $this->dsfim = $params['DSFIM'];
        $this->cdbanco = $params['CDBANCO'];
        $this->nrtipomoviment = $params['NRTIPOMOVIMENT'];
        $this->codconvenio = $params['CODCONVENIO'];
        $this->nreventom = $params['NREVENTOM'];
        $this->nreventom__label = $params['NREVENTOM__LABEL'];
        $this->nrvinculom = $params['NRVINCULOM'];
        $this->cdmatricula = $params['CDMATRICULA'];
        $this->idtpinschierarq = $params['IDTPINSCHIERARQ'];
        $this->nrpessoarespons = $params['NRPESSOARESPONS'];
        $this->nrpessoarespons__label = $params['NRPESSOARESPONS__LABEL'];
        $this->nrestrutrespons = $params['NRESTRUTRESPONS'];
        $this->idtpinscrespons = $params['IDTPINSCRESPONS'];
        $this->idutildtpgtoitem = $params['IDUTILDTPGTOITEM'];
        $this->idignoraocorrencia = $params['IDIGNORAOCORRENCIA'];

        $this->vrtotal = 0;

        $this->cdOperador = FPA::getCdOperador();
    }

    public function prepara(){
        //Verifica se já existe magnética para a competência informada
        //$magnetico = \Folha\Entities\FpaCnabmagnet::getRepository();
        $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
        $registros = $magnetico->retornaRegistros($this->competencia, $this->nrorg, $this->cdbanco, $this->nrtipomoviment, $this->cdOperador);
        $aux = 0;
        foreach($registros as $nada){ //Conta quantos registros. Espera-se 0 registros e arquivo de não teste. Caso contrário deve ser feito um retroceso.
            $aux +=1;
        }

        if($aux > 0 && $this->idteste == 'N' && $this->idgerameiomag == 'S'){
            throw new \Exception("Existe um magnético para esta competência, tipo de movimento e número do banco. Se deseja prosseguir retroceda o cálculo.");
        }

        $dtPagamento = $this->idutildtpgtoitem==='S' ? $this->dtpagamento : null;
        
        $this->vinculos = $this->retornaRegistrosVinculo($this->nrorg, $this->competencia, $this->nrmodalidade, $this->nreventom,$this->nrtipomoviment,
            $this->cdbanco, $this->dsordem, $this->dsinicio, $this->dsfim, $dtPagamento);
        
        if(empty($this->vinculos)){
            throw new \Exception("Não foram encontrados Vínculos para os parâmetros informados!");
        }

        $this->estruturas = $this->retornaRegistroEmpresa($this->nrorg,$this->nrestrutrespons, $this->competencia);

        // filtra estruturas pelo banco informado
        $this->estruturas = $this->retornaEstruturaPossuiContaCorrenteBanco($this->cdbanco, $this->estruturas);
        
        
        //VALIDAÇÕES DE DADOS
        if(!$this->estruturas[0]['CDCNPJESTRUT'])
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura sem CNPJ cadastrado.',MsgCalculo::ERRO,true);
        if(!$this->estruturas[0]['ENDERECO'])
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura sem Endereco parametrizado',MsgCalculo::ERRO,true);
        if(!$this->estruturas[0]['CDCONTCORR'] || !$this->estruturas[0]['CDAGENCIA'] || !$this->estruturas[0]['CDBANCO'])
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura sem CNPJ cadastrado.',MsgCalculo::ERRO,true);
            
        $this->nrCalculo = $this->vinculos[0]['NRCALCULOFOLHA'];
        
        foreach($this->vinculos as $vinculo){
            $this->vrtotal += $vinculo['VREVENTOFOLHA'];
            $processamento = MsgCalculo::getInstance();
            $processamento->limpaMensagens();
            $processamento->setDados($this->nrCalculo, $vinculo['NRVINCULOM'], $this->nrorg, "CNAB");
            if(!$vinculo['CDCONTCORR'] || !$vinculo['CDAGENCIA'])
                $processamento->adicionaMensagem('Erro no cadastro da conta corrente do Vínculo.',MsgCalculo::ERRO,true);
            if(!$vinculo['DSENDERECO'])
                $processamento->adicionaMensagem('Erro na descrição do endereço.',MsgCalculo::ERRO,true);
            if(!$vinculo['NRIMOVELENDERECO'])
                $processamento->adicionaMensagem('Erro no número do endereço.',MsgCalculo::ERRO,true);
            if(!$vinculo['NRCEPENDERECO'])
                $processamento->adicionaMensagem('Erro no CEP do endereço.',MsgCalculo::ERRO,true);
            if(!$vinculo['NMMUNICIPIO'])
                $processamento->adicionaMensagem('Verificar municipio do endereço.',MsgCalculo::ALERTA);
            if(!$vinculo['NMBAIRROENDERECO'])
                $processamento->adicionaMensagem('Verificar bairro do endereço.',MsgCalculo::ALERTA);
            $processamento->salvaMensagens();
        }
        
        
    }

    public function retrocede(){
        $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
        $magnetico->retroceder($this->nrorg, $this->competencia, $this->cdbanco, $this->nrtipomoviment, $this->cdOperador);
    }

    /*
     * Trunca ou completa com caracteres um registro de acordo com seu tamanho e configurações (parâmetros).
     * Também alinha a esquerda ou direita conforme '$alinhaEsquerda'
     * Otimizar este código!
     */
    protected function completaCaracter($entrada, $tamanho, $caracter, $alinhaEsquerda = true){
        $tam = strlen($entrada);    //Get no tamanho da string de entrada

        if($tam > $tamanho)         //Verifica se a string de entrada não é maior que o campo (defindo em $tamanho)
            return substr($entrada, 0, $tamanho);   //Truncando a string para $tamanho

        $caracter = substr($caracter, 0, 1);    //Garante que caracter tem apenas uma posição
        $buf = '';  //variável auxiliar para montar o registro

        if($alinhaEsquerda)     //Verifica se o texto deve ficar alinhado à esquerda
            $buf = $entrada;    //Adiciona o texto primeiro

        $tamanho = $tamanho - $tam;     //Ajuste para o loop a seguir
        for($i = 0; $i < $tamanho; ++$i){   //Adiciona os espaços, se necessário
            $buf .= $caracter;
        }

        if(!$alinhaEsquerda)    //Verifica se o texto deve ficar alinhado à direita
            $buf .= $entrada;   //Adiciona o texto após os espaços

        return $buf;
    }

    protected function getCaminhoOrg(){
        return $this->nrorg.'\\';
    }

    //Remove os caracteres BOM da string dada. Mais informações em http://en.wikipedia.org/wiki/Byte_order_mark
    public function remove_utf8_bom($text)
    {
        $bom = pack('H*','EFBBBF');
        $text = preg_replace("/^$bom/", '', $text);
        return $text;
    }
    
    public function retornaRegistroEmpresa($nrorg, $nrestruturam, $competencia){

        $sql = "
SELECT EH.NRESTRUTURAM,
   EH.NMESTRUTURAH,
   EH.NRPARCNEGOCIO,
   CC.CDBANCO,
   CC.CDAGENCIA,
   CC.CDCONTCORR,
   CC.NRTPCONTCORR,
   EH.NMRAZSOCESTRUT,
   EH.CDCNPJESTRUT,
   EPL.CDLOGRADOURO     LOGRADOURO,
   EPL.DSENDERECO       ENDERECO,
   EPL.NRIMOVELENDERECO NRO,
   EPL.NMBAIRROENDERECO BAIRRO,
   EPL.DSCOMPLEENDERECO COMPLEMENTO,
   EPL.NRCEPENDERECO    CEP,
   MUL.NMMUNICIPIO      MUNICIPIO,
   EPL.SGESTADO         UF
FROM ESTRUTURAM EM
JOIN ESTRUTURAH EH ON EH.NRORG        = EM.NRORG
                  AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
LEFT JOIN CONTCORRPARC CC ON CC.NRORG         = EM.NRORG
                         AND CC.NRPARCNEGOCIO = EH.NRPARCNEGOCIO
                         --AND CC.NRTPCONTCORR = 2 --CNAB
/*Endereço estrutura legal*/
  LEFT JOIN ENDERECOPARC EPL ON EPL.NRORG        = EM.NRORG
                          AND EPL.NRPARCNEGOCIO  = EM.NRPARCNEGOCIO
                          AND EPL.CDTIPOENDERECO = 'PRINCIPAL' 
  LEFT JOIN MUNICIPIO MUL ON ( MUL.NRORG = EPL.NRORG OR MUL.NRORG = 0 OR MUL.NRORG IS NULL )
                       AND MUL.CDPAIS      = EPL.CDPAIS
                       AND MUL.SGESTADO    = EPL.SGESTADO
                       AND MUL.CDMUNICIPIO = EPL.CDMUNICIPIO
WHERE EM.NRORG        = '{$nrorg}'
  AND EM.NRESTRUTURAM = '{$nrestruturam}'
  AND EH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC
                            FROM ESTRUTURAH
                            WHERE DTMESCOMPETENC <= '{$competencia}'
                              AND NRORG        = EM.NRORG
                              AND NRESTRUTURAM = EM.NRESTRUTURAM )";

        return $this->connection->fetchAll($sql);
    }


    /**
     * Retorna condição sql para restrição de vínculos confidenciais
     */
    private function getConfidencial(){
        $confidencial='';
        if(FPA::getRestConfidencial()==='S'){
            $confidencial = " AND VH.CONFIDENCIAL='N' ";
        }
        return $confidencial;
    }

    /**
     * Retorna os vinculos da competência ordenados por nome de vínculo vínculo
     * filtrando os parametros informados ao método  e considera a restrição de confidencial.
     *
     * @author leandroafonso
     * @params nrorg, dtcompetencia, dsordem, dsinicio, dsfim
     */
    public function retornaRegistrosVinculo($nrorg , $competencia, $nrmodalidade, $nrevento, $tipomovimento,
                                            $cdbanco , $dsOrdem , $dsInicio = null , $dsFim = null, $dtPagamento = null){
                                                
        if ($this->idignoraocorrencia == 'S') {
            $ocorrencia = null;
        } else {
            $ocorrencia = FPA::getOcorrenciaCalculo();
        }
        $sql = "SELECT VM.CDMATRICULA,
                       VM.NRVINCULOM,
                       VM.NMVINCULOM,
                       VM.DTADMISSAOVINC,
                       VM.DTRESCISAOVINC,
                       VH.NRESTRUTGEREN,
                       VH.NRESTRUTLEGAL,
                       PEH.NRCPFPESSOA,
                       EPP.CDLOGRADOURO,
                       /*Endereço vínculo*/
                       EPP.DSENDERECO,
                       EPP.NRIMOVELENDERECO,
                       EPP.NMBAIRROENDERECO,
                       EPP.DSCOMPLEENDERECO,
                       EPP.NRCEPENDERECO,
                       MUP.NMMUNICIPIO,
                       EPP.SGESTADO,
                       --ICF.IDEVENDEMONCALC,
                       SUM(ICF.VRREFFOLHA) VRREFFOLHA,
                       SUM(ICF.VREVENTOFOLHA) VREVENTOFOLHA,
                       CF.NRTIPOMOVIMENT,
                       --ICF.NRITEMCALCFOLHA,
                       MAX(CF.NRCALCULOFOLHA) NRCALCULOFOLHA,
                       TPM.NMTIPOMOVIMENT,
                       CC.NRTPCONTCORR,
                       CC.CDCONTCORR,
                       CC.CDAGENCIA,
                       CC.CDBANCO
                FROM GPE_VINCULOM VM
                JOIN GPE_VINCULOH VH
                  ON VH.NRORG      = VM.NRORG
                  AND VH.NRVINCULOM = VM.NRVINCULOM
                JOIN FPA_ITECALCFOLHA ICF
                  ON ICF.NRORG      = VM.NRORG
                  AND ICF.NRVINCULOM = VM.NRVINCULOM
                  AND ICF.DTOCORRENCIA = NVL('{$dtPagamento}', ICF.DTOCORRENCIA)
                JOIN FPA_CALCULOFOLHA CF
                  ON CF.NRORG = VM.NRORG
                  AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                  AND CF.NRTIPOMOVIMENT = '{$tipomovimento}'
                  AND CF.NRTPMODALIDCAL = '{$nrmodalidade}'
                  AND (CF.NROCORRECAL = :P_NROCORRECAL OR :P_NROCORRECAL IS NULL)
                  AND CF.DTOCORRENCIA   = '{$competencia}'
                JOIN FPA_TIPOMOVIMENT TPM
                  ON ( TPM.NRORG = VM.NRORG OR TPM.NRORG = 0 )
                  AND TPM.NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT
                JOIN GPE_PESSOA PE
                  ON  PE.NRORG    = VM.NRORG
                  AND PE.NRPESSOA = VM.NRPESSOA
                LEFT JOIN GPE_PESSOAH PEH 
                    ON PEH.NRORG    = VM.NRORG
                     AND PEH.NRPESSOA = PE.NRPESSOA
                     AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                WHERE DTMESCOMPETENC <= '{$competencia}'
                                                  AND NRORG    = PEH.NRORG
                                                  AND NRPESSOA = PEH.NRPESSOA )
                /*Endereço Pessoa*/
                LEFT JOIN ENDERECOPARC EPP ON EPP.NRORG          = PE.NRORG
                                          AND EPP.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO
                                          AND EPP.CDTIPOENDERECO = 'PRINCIPAL' 
                LEFT JOIN MUNICIPIO MUP ON ( MUP.NRORG IN ( 0, EPP.NRORG ) OR MUP.NRORG IS NULL )
                                       AND MUP.CDPAIS      = EPP.CDPAIS
                                       AND MUP.SGESTADO    = EPP.SGESTADO
                                       AND MUP.CDMUNICIPIO = EPP.CDMUNICIPIO
                JOIN CONTCORRPARC CC
                  ON  CC.NRORG = VM.NRORG
                  AND CC.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                  AND CC.IDATIVO = 'S'
                  AND CC.CDBANCO = '{$cdbanco}'
                WHERE VM.NRORG = '{$nrorg}'
                  /*AND ( VM.DTRESCISAOVINC >= '{$competencia}' OR VM.DTRESCISAOVINC IS NULL )*/ /*103753 Rafael Pickbrenner 04/12/2017*/
                  AND ICF.NREVENTOM = '{$nrevento}'
                  AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC)
                                            FROM GPE_VINCULOH
                                            WHERE DTMESCOMPETENC <= '{$competencia}'
                                              AND NRORG      = VM.NRORG
                                              AND NRVINCULOM = VM.NRVINCULOM )
                ";
                
        $params = array(
            'P_NROCORRECAL' => $ocorrencia,
        );
        
        $groupby = ' GROUP BY VM.CDMATRICULA, VM.NRVINCULOM, VM.NMVINCULOM, VM.DTADMISSAOVINC, VM.DTRESCISAOVINC, VH.NRESTRUTGEREN, VH.NRESTRUTLEGAL, PEH.NRCPFPESSOA,
                       EPP.CDLOGRADOURO, EPP.DSENDERECO, EPP.NRIMOVELENDERECO, EPP.NMBAIRROENDERECO, EPP.DSCOMPLEENDERECO, EPP.NRCEPENDERECO, MUP.NMMUNICIPIO, EPP.SGESTADO,
                       CF.NRTIPOMOVIMENT, TPM.NMTIPOMOVIMENT, CC.NRTPCONTCORR, CC.CDCONTCORR, CC.CDAGENCIA, CC.CDBANCO';

        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = ' AND VM.NMVINCULOM BETWEEN \''.$dsInicio.'\' AND \''.$dsFim.'\'';
                break;
            case 'ESTRUTURA':
                $condIntervalo = ' AND VH.NRESTRUTLEGAL BETWEEN '.$dsInicio.' AND '. (empty($dsFim) ? $dsInicio : $dsFim) .'';
                $condIntervalo .= "AND (VH.NRESTRUTGEREN = '$dsFim' OR '$dsFim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'AND VM.CDMATRICULA BETWEEN \''.$dsInicio.'\' AND \''.$dsFim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'AND VM.DTADMISSAOVINC BETWEEN \''.$dsInicio.'\' AND \''.$dsFim.'\'';
                break;
            case 'LISTA_NRESTRUTSIND':
                if($dsInicio)
                $dsInicio = is_array($dsInicio) ? implode(',',$dsInicio) : $dsInicio;
                $condIntervalo = "\r\n  AND VH.NRESTRUTLEGAL IN '$dsInicio'";
                if(isset($dsFim)){
                    $dsFim = is_array($dsFim) ? implode($dsFim, ",") : $dsFim;
                    $condIntervalo .= "\r\n  AND VH.NRESTRUTSIND IN (".$dsFim. ")";
                }
                break;
        }

        $sql.= $this->getConfidencial();
        $sql.= $condIntervalo .= $groupby .= ' ORDER BY VM.NMVINCULOM ';

        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * filtra as estruturas de acordo com o banco informado
     */
    public function retornaEstruturaPossuiContaCorrenteBanco($banco, $estruturas){
        $estruturasBanco = array();
        if(is_array($estruturas) && is_array($estruturas[0])){
            foreach($estruturas as $estrut){
                if(isset($estrut['CDBANCO']) && $estrut['CDBANCO'] == $banco){
                    array_push($estruturasBanco, $estrut);
                }
            }
        }
        return $estruturasBanco;
    }
    
    public function retornaNum($str) {
        return preg_replace("/[^0-9]/", "", $str);
    }
}
