<?php

namespace HCM\Geral\Logic;

use HCM\Geral\Logic\Apolo\PreparacaoApolo;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1000;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1005;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1020;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1030;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1040;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1050;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1200;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS1210;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2200;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2205;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2206;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2230;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2240;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2299;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2300;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2306;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2399;
use HCM\Geral\Logic\ImportacaoXmlEsocial\ImportacaoXmlS2220;
use HCM\Util\DateUtil;
use HCM\Util\FileUploader;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Symfony\Component\DependencyInjection\SimpleXMLElement;
use HCM\Geral\Logic\ImportacaoXmlEsocial\SetDsobservacao;

class ImportacaoXmlEsocial {
    
    const DSDIRMIGRACAO = "/migracao/eventos/";
    const DSDIRPENDENTES = "/pendentes";
    const DSDIRIMPORTADOS = "/importados";
    
    private $nrorg;
    private $nrorgpadrao;
    private $cdoperador;
    private $dtmescompetenc;
    private $saveLog;
    
    private $entityManager;
    
    private $status = false;
    private $messages = array();
    
    private $arrOrdemImportacao;

    protected static $queueName = "/queue/";
    
    public function __construct($nrorg, $nrorgpadrao, $cdoperador, $dtmescompetenc, $saveLog=true) {
        $this->nrorg             = $nrorg;
        $this->nrorgpadrao       = $nrorgpadrao;
        $this->cdoperador        = $cdoperador;
        $this->dtmescompetenc    = $dtmescompetenc;
        $this->saveLog           = $saveLog;
        $this->entityManager     = Util::getEntityManager();
        
        $this->arrOrdemImportacao = array(
            1 => 'S-1000',
            2 => 'S-1005',
            3 => 'S-1020',
            4 => 'S-1030',
            5 => 'S-1040',
            6 => 'S-1050',
            7 => 'S-2200',
            8 => 'S-2205',
            9 => 'S-2206',
            10 => 'S-2230',
            11 => 'S-2300',
            12 => 'S-2306',
            13 => 'S-2299',
            14 => 'S-1200',
            15 => 'S-1210',
            16 => 'S-2220',
            17 => 'S-2240',
            98 => 'S-2299',
            99 => 'S-2399',
        );
    }
    
    private function getNmEventoXmlFile($xmlFile) {
        if($xmlFile->evtInfoEmpregador){
            return 'S-1000';
        }
        else if ($xmlFile->evtTabEstab){
            return 'S-1005';
        }
        else if ($xmlFile->evtTabLotacao){
            return 'S-1020';
        }
        else if ($xmlFile->evtTabCargo){
            return 'S-1030';
        }
        else if ($xmlFile->evtTabFuncao){
            return 'S-1040';
        }
        else if ($xmlFile->evtTabHorTur){
            return 'S-1050';
        }
        else if ($xmlFile->evtAdmissao){
            return 'S-2200';
        }
        else if ($xmlFile->evtAltCadastral){
            return 'S-2205';
        }
        else if ($xmlFile->evtAltContratual){
            return 'S-2206';
        }
        else if ($xmlFile->evtMonit){
          return 'S-2220';
        }
        else if ($xmlFile->evtAfastTemp){
            return 'S-2230';
        }
        else if ($xmlFile->evtTSVInicio){
            return 'S-2300';
        }
        else if ($xmlFile->evtTSVAltContr){
            return 'S-2306';
        }
        else if ($xmlFile->evtRemun){
            return 'S-1200';
        }
        else if ($xmlFile->evtPgtos){
            return 'S-1210';
        }
        else if ($xmlFile->evtDeslig){
            return 'S-2299';
        }
        else if ($xmlFile->evtTSVTermino){
            return 'S-2399';
        } else if ($xmlFile->evtExpRisco) {
            return 'S-2240';
        }
        else{
            return false;
        }
    }
    
    private function getIdEventoXmlFile($xmlFile, $nmEvento) {
        if($nmEvento == 'S-1000'){
            return (string) $xmlFile->evtInfoEmpregador["Id"];
        }
        else if ($nmEvento == 'S-1005'){
            return (string) $xmlFile->evtTabEstab["Id"];
        }
        else if ($nmEvento == 'S-1020'){
            return (string) $xmlFile->evtTabLotacao["Id"];
        }
        else if ($nmEvento == 'S-1030'){
            return (string) $xmlFile->evtTabCargo["Id"];
        }
        else if ($nmEvento == 'S-1040'){
            return (string) $xmlFile->evtTabFuncao["Id"];
        }
        else if ($nmEvento == 'S-1050'){
            return (string) $xmlFile->evtTabHorTur["Id"];
        }
        else if ($nmEvento == 'S-2200'){
            return (string) $xmlFile->evtAdmissao["Id"];
        }
        else if ($nmEvento == 'S-2205'){
            return (string) $xmlFile->evtAltCadastral["Id"];
        }
        else if ($nmEvento == 'S-2206'){
            return (string) $xmlFile->evtAltContratual["Id"];
        }
        else if ($nmEvento == 'S-2220'){
          return (string) $xmlFile->evtMonit["Id"];
        }
        else if ($nmEvento == 'S-2230'){
            return (string) $xmlFile->evtAfastTemp["Id"];
        }
        else if ($nmEvento == 'S-2300'){
            return (string) $xmlFile->evtTSVInicio["Id"];
        }
        else if ($nmEvento == 'S-2306'){
            return (string) $xmlFile->evtTSVAltContr["Id"];
        }
        else if ($nmEvento == 'S-1200'){
            return (string) $xmlFile->evtRemun["Id"];
        }
        else if ($nmEvento == 'S-1210'){
            return (string) $xmlFile->evtPgtos["Id"];
        }
        else if ($nmEvento == 'S-2299'){
            return (string) $xmlFile->evtDeslig["Id"];
        }
        else if ($nmEvento == 'S-2399'){
            return (string) $xmlFile->evtTSVTermino["Id"];
        } else if ($nmEvento == 'S-2240') {
            return (string) $xmlFile->evtExpRisco["Id"];
        }
        else{
            return null;
        }
    }
    
    private function getVersaoEventoXmlFile($xmlFile) {
        $arrNamespace = explode("/",current($xmlFile->getNamespaces()));
        
        $versao = end($arrNamespace);
        
        if (strpos($versao, 'S')) {
          return str_replace('_','.',str_replace('v_','',$versao)); 
        }

        return str_replace('_','.',str_replace('v','',$versao));
    }


    private function saveXmlOnFileServer($fileContent, $fileName, $customPath='') {
        $fileFieldName = "DSPATH";
        $row = array(
            "DSPATH" => array(
                array(
                    "name"               => $fileName,
                    "lastModified"       => 1507122512763,
                    "lastModifiedDate"   => DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA),
                    "webkitRelativePath" => "",
                    "size"               => strlen($fileContent),
                    "type"               => "application/xml",
                    "b64File"            => "data:application/xml;base64," . (string)base64_encode($fileContent),
                ),
            )
        );
        
        FileUploader::uploadFilesInRow($row, $fileFieldName, $customPath);
        
        $filePath = $row['DSPATH'][0]['path'];
        
        return $filePath;
    }
    
    private function getArquivosPendentesEvento($nmevento) {
        $arquivos = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findBy(
            array('nrorg' => $this->nrorg, 'nmevento' => $nmevento, 'idstatus' => ['P','A']),
            array('nmarquivoxml' => 'asc')
        );
        return $arquivos;
    }
    
    public function getStatus() {
        return $this->status;
    }
    
    public function getMessages() {
        return $this->messages;
    }
    
    /**
     * Function para montar o xml e retornar exceções do construtor
     */
    private function getData($fileContent) {

        set_error_handler(function($errno, $errstr, $errfile, $errline) {
            throw new \Exception($errstr, $errno);
        });
    
        try {
            //Remove caracter BOM do texto (Erro ocorrido nos arquivos da Sumatex pois estava causando erro na conversão utf8)
            $bom = pack('H*','EFBBBF');
            $fileContent = preg_replace("/^$bom/", '', $fileContent);
            
            //Incluída verificação com função utf8 somente para identificar se o arquivo não está com caracteres inválidos para a função simplexml_load_string
            if($fileContent != utf8_encode($fileContent)){
                $fileContent = utf8_encode($fileContent);
            }
            
            //$xml = new SimpleXMLElement($fileContent);
            $xml = simplexml_load_string($fileContent);
        }catch(Exception $e) {
            restore_error_handler();
            throw $e;
        }
    
        return $xml;
    }
    
    public function setMigracaoXml($tmpFile, $fileName){
        try{
            $this->entityManager->beginTransaction();
            
            $fileContent = file_get_contents($tmpFile);
            
            $xml = $this->getData($fileContent);
            
            //Seta os namespaces do envelope soap e versão 1.1.0 do envio de lote do esocial
            $xml->registerXPathNamespace("soapenv", "http://schemas.xmlsoap.org/soap/envelope/");
            $xml->registerXPathNamespace("v1", "http://www.esocial.gov.br/servicos/empregador/lote/eventos/envio/v1_1_0");
            
            $body = $xml->xpath("//soapenv:Body");
            $loteEventos = $xml->xpath("//v1:loteEventos");
            
            //Verifica se existe o body do envolope pra saber se é um lote ou o xml específico de um evento
            if (empty($body)) {
                if ($xml->envioLoteEventos) {
                    //Em caso de lote percorre os eventos do lote inserindo um a um
                    foreach ($xml->envioLoteEventos->eventos->evento as $evento) {
                        $this->newMigracaoXml($evento->eSocial, $fileName);
                    }
                } else if ($xml->retornoProcessamentoDownload) {
                    $this->newMigracaoXml($xml->retornoProcessamentoDownload->evento->eSocial, $fileName);
                } else {
                    //Faz o insert caso já seja o xml de um evento
                    $this->newMigracaoXml($xml, $fileName);
                }
            } else {
                if (!empty($loteEventos)) {
                    //Em caso de lote percorre os eventos do lote inserindo um a um
                    foreach ($loteEventos[0]->eSocial->envioLoteEventos->eventos->evento as $evento) {
                        $this->newMigracaoXml($evento->eSocial, $fileName);
                    }
                } else {
                    throw new \Exception("Arquivo $fileName não corresponde à um evento para migração", 10);
                }
            }
            
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function newMigracaoXml($xml,$fileName){
        
        $nmevento = $this->getNmEventoXmlFile($xml);
        $idevento = $this->getIdEventoXmlFile($xml, $nmevento);
        $dsversao = $this->getVersaoEventoXmlFile($xml);
        
        if($nmevento === false){
            throw new \Exception("Arquivo $fileName não corresponde à um evento para migração", 10);
        }
        
        /*Salva o arquivo no fileServer no pasta padrão para os arquivos pendentes das migrações*/
        $customPath = self::DSDIRMIGRACAO . $nmevento . self::DSDIRPENDENTES;
        $filePath = $this->saveXmlOnFileServer($xml->asXML(), $fileName, $customPath);
        
        /*Remove a extensão do arquivo e acrescenta o Id do evento ao nome do arquivo recolocando a extensão .xml*/
        $nmArquivo = preg_replace('/\\.[^.\\s]{3,4}$/', '', $fileName) . " (" . $idevento . ").xml";
        
        $esoMigracaoXml = Repositories::ESO_MIGRACAOXML;
            
        /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoMigracaoxml $newMigracaoXml */
        $newMigracaoXml = new $esoMigracaoXml();
        $newMigracaoXml->setNrorg($this->nrorg);
        $newMigracaoXml->setNmevento($nmevento);
        $newMigracaoXml->setDsversao($dsversao);
        $newMigracaoXml->setNmarquivoxml($nmArquivo);
        $newMigracaoXml->setDsarquivoxml($filePath);
        $newMigracaoXml->setIdstatus('P');
        $newMigracaoXml->setIdativo('S');
        $newMigracaoXml->setDtinclusao(DateUtil::getDataAtual());
        $newMigracaoXml->setCdoperinclusao($this->cdoperador);
        $newMigracaoXml->setNrorginclusao($this->nrorg);
        $newMigracaoXml->setNrmigracaoxml(\HCM\Util\NovoCodigo::geraCodigo($this->nrorg, 'ESO_MIGRACAOXML', 1, 12, true));
        $this->entityManager->persist($newMigracaoXml);
        $this->entityManager->flush();
        
        return $newMigracaoXml;
    }
    
    public function validaEventosPrecedentesPendentes($migracaoXml) {
        $pendentes = array();
        $nrordem = 99;
        
        foreach($this->arrOrdemImportacao as $ordem => $evento) {
            if ($evento == $migracaoXml->getNmevento()) {
                $nrordem = $ordem;
            }
        }
        
        for($i = 1; $i < $nrordem; $i++) {
            if(!empty($this->getArquivosPendentesEvento($this->arrOrdemImportacao[$i]))) {
                $pendentes[] = $this->arrOrdemImportacao[$i];
                $result = false;
            }
        }
        
        if (!empty($pendentes)) {
            $msgPendentes = 'Existem arquivos pendentes para o(s) evento(s) ' . implode(',',$pendentes);
            
            $this->messages[] = $msgPendentes;
            if ($this->saveLog){
                $this->saveLogMigracaoXml($migracaoXml, array($msgPendentes));
            }
                
            return false;
        }
        
        return true;
    }
    
    public function agendarMigracaoXmlPorEvento($nmevento, $paramsHcmParametros) {
        try {
            $paramsApolo = FPA::getParamsApolo();
        
            $url = $paramsApolo['URL'];
            $port = $paramsApolo['PORT'];
            $user = $paramsApolo['USER'];
            $password = $paramsApolo['PASSWORD'];
            $broke = $paramsApolo['BROKE'];
            
            $preparaFilaApolo = new PreparacaoApolo($this->nrorg, $url, $port, $user, $password, $broke);
            
            $arquivos = $this->getArquivosPendentesEvento($nmevento);
            
            foreach($arquivos as $registro) {
                $registros[] = $registro->getNrmigracaoxml();
            }
            
            $params = array(
                'REGISTROS' => $registros,
                "PARAMSHCM" => $paramsHcmParametros
            );
            
            $result = $preparaFilaApolo->sendMessage('MigracaoXml', $params);
            $this->messages[] = $result['MSG'];
            $this->status = true;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function importarMigracaoXmlPorEvento($nmevento) {
        $arquivos = $this->getArquivosPendentesEvento($nmevento);
        foreach($arquivos as $migracaoXml) {
            $this->importarMigracaoXml($migracaoXml);
        }
    }
    
    public function importarMigracaoXml($migracaoXml) {
        set_time_limit(0);
        ini_set('memory_limit', '2048M');
        if ($migracaoXml->getIdstatus() == 'C') {
            $this->messages[] = 'Arquivo já importado (' . $migracaoXml->getNmarquivoxml() . ')';
        } else {
            $xmlFileDir = Util::getFileServerUrl() . $migracaoXml->getDsarquivoxml();
            $xml = file_get_contents($xmlFileDir);
            if ($this->validaEventosPrecedentesPendentes($migracaoXml)) {
                try {
                    //Seta o registro de migração para status em andamento para saber q este foi iniciado
                    $migracaoXml->setIdstatus('A');
                    $this->entityManager->persist($migracaoXml);
                    $this->entityManager->flush();
                    switch($migracaoXml->getNmEvento()) {
                        case 'S-1000':
                            $result = $this->importarXmlS1000($xml);
                            break;
                        case 'S-1005':
                            $result = $this->importarXmlS1005($xml);
                            break;
                        case 'S-1020':
                            $result = $this->importarXmlS1020($xml);
                            break;
                        case 'S-1030':
                            $result = $this->importarXmlS1030($xml);
                            break;
                        case 'S-1040':
                            $result = $this->importarXmlS1040($xml);
                            break;
                        case 'S-1050':
                            $result = $this->importarXmlS1050($xml);
                            break;
                        case 'S-2200':
                            $result = $this->importarXmlS2200($xml);
                            break;
                        case 'S-2205':
                            $result = $this->importarXmlS2205($xml);
                            break;
                        case 'S-2206':
                            $result = $this->importarXmlS2206($xml);
                            break;
                        case 'S-2220':
                            $result = $this->importarXmlS2220($xml);
                            break;
                        case 'S-2230':
                            $result = $this->importarXmlS2230($xml);
                            break;
                        case 'S-2300':
                            $result = $this->importarXmlS2300($xml);
                            break;
                        case 'S-2306':
                            $result = $this->importarXmlS2306($xml);
                            break;
                        case 'S-1200':
                            $result = $this->importarXmlS1200($xml);
                            break;
                        case 'S-1210':
                            $result = $this->importarXmlS1210($xml);
                            break;
                        case 'S-2299':
                            $result = $this->importarXmlS2299($xml);
                            break;
                        case 'S-2399':
                            $result = $this->importarXmlS2399($xml);
                            break;
                        case 'S-2240':
                            $result = $this->importarXmlS2240($xml);
                            break;
                        default:
                            $result['status'] = false;
                    }
                } catch(\Exception $e) {
                    if ($e->getCode() == 10) {
                        $result['status'] = false;
                        $result['message'] = $e->getMessage();
                    } else {
                        throw $e;
                    }
                }
            } else {
                $result['status'] = false;
                $result['message'] = "Não foi possível realizar a importação";
            }
            
            if ($result['status'] == true) {
                $oldFilePath = $migracaoXml->getDsarquivoxml();
                $fileName = explode('/',$oldFilePath);
                $fileName = end($fileName);
                
                $fileContent = file_get_contents(Util::getFileServerUrl() . $oldFilePath);
                
                $customPath = self::DSDIRMIGRACAO . $migracaoXml->getNmevento() . self::DSDIRIMPORTADOS;
                $newfilePath = $this->saveXmlOnFileServer($fileContent, $fileName, $customPath);
                $migracaoXml->setDsarquivoxml($newfilePath);
                
                $migracaoXml->setIdstatus('C');
                $migracaoXml->setDtultatu(DateUtil::getDataAtual());
                $migracaoXml->setCdoperultatu($this->cdoperador);
                $migracaoXml->setNrorgultatu($this->nrorg);
                
                $this->entityManager->persist($migracaoXml);
                $this->entityManager->flush();
                
                $this->status = true;
                foreach ($result['messages'] as $message) {
                    $this->messages[] = $message . ' (' . $migracaoXml->getNmarquivoxml() . ')';
                }
            
                if ($this->saveLog){
                    $this->saveLogMigracaoXml($migracaoXml, $result['messages']);
                }
                
                FileUploader::deleteFile($oldFilePath);
            } else {
                $this->messages[] = $result['message'] . ' (' . $migracaoXml->getNmarquivoxml() . ')';
            
                if ($this->saveLog){
                    $this->saveLogMigracaoXml($migracaoXml, array($result['message']));
                }
            }
        }
    }
    
    public function importarXmlS1000($xml){
        $importacaoXml = new ImportacaoXmlS1000($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1005($xml){
        $importacaoXml = new ImportacaoXmlS1005($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1020($xml){
        $importacaoXml = new ImportacaoXmlS1020($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1030($xml){
        $importacaoXml = new ImportacaoXmlS1030($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1040($xml){
        $importacaoXml = new ImportacaoXmlS1040($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1050($xml){
        $importacaoXml = new ImportacaoXmlS1050($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2200($xml){
        $importacaoXml = new ImportacaoXmlS2200($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2205($xml){
        $importacaoXml = new ImportacaoXmlS2205($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2206($xml){
        $importacaoXml = new ImportacaoXmlS2206($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2230($xml){
        $importacaoXml = new ImportacaoXmlS2230($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2300($xml){
        $importacaoXml = new ImportacaoXmlS2300($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2306($xml){
        $importacaoXml = new ImportacaoXmlS2306($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1200($xml){
        $importacaoXml = new ImportacaoXmlS1200($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS1210($xml){
        $importacaoXml = new ImportacaoXmlS1210($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }
    
    public function importarXmlS2299($xml){
        $importacaoXml = new ImportacaoXmlS2299($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }

    public function importarXmlS2399($xml){
        $importacaoXml = new ImportacaoXmlS2399($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }

    public function importarXmlS2240($xml){
        $importacaoXml = new ImportacaoXmlS2240($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
        return $importacaoXml->importarXml();
    }

    public function importarXmlS2220($xml){
      $importacaoXml = new ImportacaoXmlS2220($this->nrorg, $this->nrorgpadrao, $this->dtmescompetenc, $this->cdoperador, $xml);
      return $importacaoXml->importarXml();
    }
    
    public function saveLogMigracaoXml($migracaoXml, $messages) {
        foreach ($messages as $message) {
            $esoLogmigracaoXml = Repositories::ESO_LOGMIGRACAOXML;
            
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoLogmigracaoxml $newLogmigracaoXml */
            $newLogmigracaoXml = new $esoLogmigracaoXml();
            $newLogmigracaoXml->setNrorg($this->nrorg);
            $newLogmigracaoXml->setNrmigracaoxml($migracaoXml->getNrmigracaoxml());
            $newLogmigracaoXml->setDsmensagem($message);
            $newLogmigracaoXml->setIdativo('S');
            $newLogmigracaoXml->setDtinclusao(DateUtil::getDataAtual());
            $newLogmigracaoXml->setCdoperinclusao($this->cdoperador);
            $newLogmigracaoXml->setNrorginclusao($this->nrorg);
            $newLogmigracaoXml->setNrLogmigracaoxml(\HCM\Util\NovoCodigo::geraCodigo($this->nrorg, 'ESO_LOGMIGRACAOXML', 1, 12, true));
            
            $this->entityManager->persist($newLogmigracaoXml);
            $this->entityManager->flush();
        }
    }
    
}
